@@ -2,6 +2,7 @@ import re
 from datetime import timedelta
 from pathlib import Path
 
+import joblib
 import pandas as pd
 import pytest
 from arrow import Arrow
@@ -12,17 +13,18 @@ from freqtrade.data import history
 from freqtrade.data.btanalysis import (get_latest_backtest_filename, load_backtest_data,
                                        load_backtest_stats)
 from freqtrade.edge import PairInfo
-from freqtrade.enums import SellType
+from freqtrade.enums import ExitType
 from freqtrade.optimize.optimize_reports import (_get_resample_from_period, generate_backtest_stats,
                                                  generate_daily_stats, generate_edge_table,
-                                                 generate_pair_metrics,
+                                                 generate_exit_reason_stats, generate_pair_metrics,
                                                  generate_periodic_breakdown_stats,
-                                                 generate_sell_reason_stats,
                                                  generate_strategy_comparison,
                                                  generate_trading_stats, show_sorted_pairlist,
+                                                 store_backtest_signal_candles,
                                                  store_backtest_stats, text_table_bt_results,
-                                                 text_table_sell_reason, text_table_strategy)
+                                                 text_table_exit_reason, text_table_strategy)
 from freqtrade.resolvers.strategy_resolver import StrategyResolver
+from tests.conftest import CURRENT_TEST_STRATEGY
 from tests.data.test_history import _backup_file, _clean_test_file
 
 
@@ -54,7 +56,7 @@ def test_text_table_bt_results():
 
 
 def test_generate_backtest_stats(default_conf, testdatadir, tmpdir):
-    default_conf.update({'strategy': 'StrategyTestV2'})
+    default_conf.update({'strategy': CURRENT_TEST_STRATEGY})
     StrategyResolver.load_strategy(default_conf)
 
     results = {'DefStrat': {
@@ -74,9 +76,10 @@ def test_generate_backtest_stats(default_conf, testdatadir, tmpdir):
                                  "close_rate": [0.002546, 0.003014, 0.003103, 0.003217],
                                  "trade_duration": [123, 34, 31, 14],
                                  "is_open": [False, False, False, True],
+                                 "is_short": [False, False, False, False],
                                  "stake_amount": [0.01, 0.01, 0.01, 0.01],
-                                 "sell_reason": [SellType.ROI, SellType.STOP_LOSS,
-                                                 SellType.ROI, SellType.FORCE_SELL]
+                                 "exit_reason": [ExitType.ROI, ExitType.STOP_LOSS,
+                                                 ExitType.ROI, ExitType.FORCE_EXIT]
                                  }),
         'config': default_conf,
         'locks': [],
@@ -125,9 +128,10 @@ def test_generate_backtest_stats(default_conf, testdatadir, tmpdir):
              "close_rate": [0.002546, 0.003014, 0.0032903, 0.003217],
              "trade_duration": [123, 34, 31, 14],
              "is_open": [False, False, False, True],
+             "is_short": [False, False, False, False],
              "stake_amount": [0.01, 0.01, 0.01, 0.01],
-             "sell_reason": [SellType.ROI, SellType.ROI,
-                             SellType.STOP_LOSS, SellType.FORCE_SELL]
+             "exit_reason": [ExitType.ROI, ExitType.ROI,
+                             ExitType.STOP_LOSS, ExitType.FORCE_EXIT]
              }),
         'config': default_conf,
         'locks': [],
@@ -188,7 +192,7 @@ def test_store_backtest_stats(testdatadir, mocker):
 
     assert dump_mock.call_count == 3
     assert isinstance(dump_mock.call_args_list[0][0][0], Path)
-    assert str(dump_mock.call_args_list[0][0][0]).startswith(str(testdatadir/'backtest-result'))
+    assert str(dump_mock.call_args_list[0][0][0]).startswith(str(testdatadir / 'backtest-result'))
 
     dump_mock.reset_mock()
     filename = testdatadir / 'testresult.json'
@@ -199,6 +203,62 @@ def test_store_backtest_stats(testdatadir, mocker):
     assert str(dump_mock.call_args_list[0][0][0]).startswith(str(testdatadir / 'testresult'))
 
 
+def test_store_backtest_candles(testdatadir, mocker):
+
+    dump_mock = mocker.patch('freqtrade.optimize.optimize_reports.file_dump_joblib')
+
+    candle_dict = {'DefStrat': {'UNITTEST/BTC': pd.DataFrame()}}
+
+    # mock directory exporting
+    store_backtest_signal_candles(testdatadir, candle_dict)
+
+    assert dump_mock.call_count == 1
+    assert isinstance(dump_mock.call_args_list[0][0][0], Path)
+    assert str(dump_mock.call_args_list[0][0][0]).endswith(str('_signals.pkl'))
+
+    dump_mock.reset_mock()
+    # mock file exporting
+    filename = Path(testdatadir / 'testresult')
+    store_backtest_signal_candles(filename, candle_dict)
+    assert dump_mock.call_count == 1
+    assert isinstance(dump_mock.call_args_list[0][0][0], Path)
+    # result will be testdatadir / testresult-<timestamp>_signals.pkl
+    assert str(dump_mock.call_args_list[0][0][0]).endswith(str('_signals.pkl'))
+    dump_mock.reset_mock()
+
+
+def test_write_read_backtest_candles(tmpdir):
+
+    candle_dict = {'DefStrat': {'UNITTEST/BTC': pd.DataFrame()}}
+
+    # test directory exporting
+    stored_file = store_backtest_signal_candles(Path(tmpdir), candle_dict)
+    scp = open(stored_file, "rb")
+    pickled_signal_candles = joblib.load(scp)
+    scp.close()
+
+    assert pickled_signal_candles.keys() == candle_dict.keys()
+    assert pickled_signal_candles['DefStrat'].keys() == pickled_signal_candles['DefStrat'].keys()
+    assert pickled_signal_candles['DefStrat']['UNITTEST/BTC'] \
+        .equals(pickled_signal_candles['DefStrat']['UNITTEST/BTC'])
+
+    _clean_test_file(stored_file)
+
+    # test file exporting
+    filename = Path(tmpdir / 'testresult')
+    stored_file = store_backtest_signal_candles(filename, candle_dict)
+    scp = open(stored_file, "rb")
+    pickled_signal_candles = joblib.load(scp)
+    scp.close()
+
+    assert pickled_signal_candles.keys() == candle_dict.keys()
+    assert pickled_signal_candles['DefStrat'].keys() == pickled_signal_candles['DefStrat'].keys()
+    assert pickled_signal_candles['DefStrat']['UNITTEST/BTC'] \
+        .equals(pickled_signal_candles['DefStrat']['UNITTEST/BTC'])
+
+    _clean_test_file(stored_file)
+
+
 def test_generate_pair_metrics():
 
     results = pd.DataFrame(
@@ -226,7 +286,7 @@ def test_generate_pair_metrics():
 
 def test_generate_daily_stats(testdatadir):
 
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     bt_data = load_backtest_data(filename)
     res = generate_daily_stats(bt_data)
     assert isinstance(res, dict)
@@ -246,7 +306,7 @@ def test_generate_daily_stats(testdatadir):
 
 
 def test_generate_trading_stats(testdatadir):
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     bt_data = load_backtest_data(filename)
     res = generate_trading_stats(bt_data)
     assert isinstance(res, dict)
@@ -262,7 +322,7 @@ def test_generate_trading_stats(testdatadir):
     assert res['losses'] == 0
 
 
-def test_text_table_sell_reason():
+def test_text_table_exit_reason():
 
     results = pd.DataFrame(
         {
@@ -273,12 +333,12 @@ def test_text_table_sell_reason():
             'wins': [2, 0, 0],
             'draws': [0, 0, 0],
             'losses': [0, 0, 1],
-            'sell_reason': [SellType.ROI, SellType.ROI, SellType.STOP_LOSS]
+            'exit_reason': [ExitType.ROI, ExitType.ROI, ExitType.STOP_LOSS]
         }
     )
 
     result_str = (
-        '|   Sell Reason |   Sells |   Win  Draws  Loss  Win% |   Avg Profit % |   Cum Profit % |'
+        '|   Exit Reason |   Exits |   Win  Draws  Loss  Win% |   Avg Profit % |   Cum Profit % |'
         '   Tot Profit BTC |   Tot Profit % |\n'
         '|---------------+---------+--------------------------+----------------+----------------+'
         '------------------+----------------|\n'
@@ -288,9 +348,9 @@ def test_text_table_sell_reason():
         '             -0.2 |             -5 |'
     )
 
-    sell_reason_stats = generate_sell_reason_stats(max_open_trades=2,
+    exit_reason_stats = generate_exit_reason_stats(max_open_trades=2,
                                                    results=results)
-    assert text_table_sell_reason(sell_reason_stats=sell_reason_stats,
+    assert text_table_exit_reason(exit_reason_stats=exit_reason_stats,
                                   stake_currency='BTC') == result_str
 
 
@@ -305,23 +365,23 @@ def test_generate_sell_reason_stats():
             'wins': [2, 0, 0],
             'draws': [0, 0, 0],
             'losses': [0, 0, 1],
-            'sell_reason': [SellType.ROI.value, SellType.ROI.value, SellType.STOP_LOSS.value]
+            'exit_reason': [ExitType.ROI.value, ExitType.ROI.value, ExitType.STOP_LOSS.value]
         }
     )
 
-    sell_reason_stats = generate_sell_reason_stats(max_open_trades=2,
+    exit_reason_stats = generate_exit_reason_stats(max_open_trades=2,
                                                    results=results)
-    roi_result = sell_reason_stats[0]
-    assert roi_result['sell_reason'] == 'roi'
+    roi_result = exit_reason_stats[0]
+    assert roi_result['exit_reason'] == 'roi'
     assert roi_result['trades'] == 2
     assert pytest.approx(roi_result['profit_mean']) == 0.15
     assert roi_result['profit_mean_pct'] == round(roi_result['profit_mean'] * 100, 2)
     assert pytest.approx(roi_result['profit_mean']) == 0.15
     assert roi_result['profit_mean_pct'] == round(roi_result['profit_mean'] * 100, 2)
 
-    stop_result = sell_reason_stats[1]
+    stop_result = exit_reason_stats[1]
 
-    assert stop_result['sell_reason'] == 'stop_loss'
+    assert stop_result['exit_reason'] == 'stop_loss'
     assert stop_result['trades'] == 1
     assert pytest.approx(stop_result['profit_mean']) == -0.1
     assert stop_result['profit_mean_pct'] == round(stop_result['profit_mean'] * 100, 2)
@@ -330,7 +390,7 @@ def test_generate_sell_reason_stats():
 
 
 def test_text_table_strategy(testdatadir):
-    filename = testdatadir / "backtest-result_multistrat.json"
+    filename = testdatadir / "backtest_results/backtest-result_multistrat.json"
     bt_res_data = load_backtest_stats(filename)
 
     bt_res_data_comparison = bt_res_data.pop('strategy_comparison')
@@ -362,7 +422,7 @@ def test_generate_edge_table():
 
 
 def test_generate_periodic_breakdown_stats(testdatadir):
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     bt_data = load_backtest_data(filename).to_dict(orient='records')
 
     res = generate_periodic_breakdown_stats(bt_data, 'day')
@@ -390,13 +450,13 @@ def test__get_resample_from_period():
 
 
 def test_show_sorted_pairlist(testdatadir, default_conf, capsys):
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     bt_data = load_backtest_stats(filename)
     default_conf['backtest_show_pair_list'] = True
 
     show_sorted_pairlist(default_conf, bt_data)
 
     out, err = capsys.readouterr()
-    assert 'Pairs for Strategy StrategyTestV2: \n[' in out
+    assert 'Pairs for Strategy StrategyTestV3: \n[' in out
     assert 'TOTAL' not in out
     assert '"ETH/BTC",  // ' in out
