@@ -9,7 +9,7 @@ import pytest
 import time_machine
 
 from freqtrade.constants import AVAILABLE_PAIRLISTS
-from freqtrade.enums import RunMode
+from freqtrade.enums import CandleType, RunMode
 from freqtrade.exceptions import OperationalException
 from freqtrade.persistence import Trade
 from freqtrade.plugins.pairlist.pairlist_helpers import expand_pairlist
@@ -491,11 +491,11 @@ def test_VolumePairList_whitelist_gen(mocker, whitelist_conf, shitcoinmarkets, t
     ohlcv_history_high_vola.loc[ohlcv_history_high_vola.index == 1, 'close'] = 0.00090
 
     ohlcv_data = {
-        ('ETH/BTC', '1d'): ohlcv_history,
-        ('TKN/BTC', '1d'): ohlcv_history,
-        ('LTC/BTC', '1d'): pd.concat([ohlcv_history, ohlcv_history]),
-        ('XRP/BTC', '1d'): ohlcv_history,
-        ('HOT/BTC', '1d'): ohlcv_history_high_vola,
+        ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('LTC/BTC', '1d', CandleType.SPOT): pd.concat([ohlcv_history, ohlcv_history]),
+        ('XRP/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('HOT/BTC', '1d', CandleType.SPOT): ohlcv_history_high_vola,
     }
 
     mocker.patch('freqtrade.exchange.Exchange.exchange_has', MagicMock(return_value=True))
@@ -587,10 +587,10 @@ def test_VolumePairList_whitelist_gen(mocker, whitelist_conf, shitcoinmarkets, t
     ([{"method": "VolumePairList", "number_assets": 5, "sort_key": "quoteVolume",
        "lookback_timeframe": "1d", "lookback_period": 1, "refresh_period": 86400}],
      "BTC", "binance", ['LTC/BTC', 'ETH/BTC', 'TKN/BTC', 'XRP/BTC', 'HOT/BTC']),
-    # expecting pairs from default tickers, because 1h candles are not available
+    # expecting pairs as input, because 1h candles are not available
     ([{"method": "VolumePairList", "number_assets": 5, "sort_key": "quoteVolume",
        "lookback_timeframe": "1h", "lookback_period": 2, "refresh_period": 3600}],
-     "BTC", "binance", ['ETH/BTC', 'TKN/BTC', 'LTC/BTC', 'HOT/BTC', 'FUEL/BTC']),
+     "BTC", "binance", ['ETH/BTC', 'LTC/BTC', 'NEO/BTC', 'TKN/BTC', 'XRP/BTC']),
     # ftx data is already in Quote currency, therefore won't require conversion
     ([{"method": "VolumePairList", "number_assets": 5, "sort_key": "quoteVolume",
        "lookback_timeframe": "1d", "lookback_period": 1, "refresh_period": 86400}],
@@ -619,11 +619,11 @@ def test_VolumePairList_range(mocker, whitelist_conf, shitcoinmarkets, tickers,
     mocker.patch('freqtrade.exchange.ftx.Ftx.market_is_tradable', return_value=True)
 
     ohlcv_data = {
-        ('ETH/BTC', '1d'): ohlcv_history,
-        ('TKN/BTC', '1d'): ohlcv_history,
-        ('LTC/BTC', '1d'): ohlcv_history_medium_volume,
-        ('XRP/BTC', '1d'): ohlcv_history_high_vola,
-        ('HOT/BTC', '1d'): ohlcv_history_high_volume,
+        ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('LTC/BTC', '1d', CandleType.SPOT): ohlcv_history_medium_volume,
+        ('XRP/BTC', '1d', CandleType.SPOT): ohlcv_history_high_vola,
+        ('HOT/BTC', '1d', CandleType.SPOT): ohlcv_history_high_volume,
     }
 
     mocker.patch('freqtrade.exchange.Exchange.exchange_has', MagicMock(return_value=True))
@@ -937,9 +937,9 @@ def test_agefilter_min_days_listed_too_large(mocker, default_conf, markets, tick
 def test_agefilter_caching(mocker, markets, whitelist_conf_agefilter, tickers, ohlcv_history):
     with time_machine.travel("2021-09-01 05:00:00 +00:00") as t:
         ohlcv_data = {
-            ('ETH/BTC', '1d'): ohlcv_history,
-            ('TKN/BTC', '1d'): ohlcv_history,
-            ('LTC/BTC', '1d'): ohlcv_history,
+            ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('LTC/BTC', '1d', CandleType.SPOT): ohlcv_history,
         }
         mocker.patch.multiple(
             'freqtrade.exchange.Exchange',
@@ -961,10 +961,10 @@ def test_agefilter_caching(mocker, markets, whitelist_conf_agefilter, tickers, o
         assert freqtrade.exchange.refresh_latest_ohlcv.call_count == 2
 
         ohlcv_data = {
-            ('ETH/BTC', '1d'): ohlcv_history,
-            ('TKN/BTC', '1d'): ohlcv_history,
-            ('LTC/BTC', '1d'): ohlcv_history,
-            ('XRP/BTC', '1d'): ohlcv_history.iloc[[0]],
+            ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('LTC/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('XRP/BTC', '1d', CandleType.SPOT): ohlcv_history.iloc[[0]],
         }
         mocker.patch('freqtrade.exchange.Exchange.refresh_latest_ohlcv', return_value=ohlcv_data)
         freqtrade.pairlists.refresh_pairlist()
@@ -982,10 +982,10 @@ def test_agefilter_caching(mocker, markets, whitelist_conf_agefilter, tickers, o
         t.move_to("2021-09-03 01:00:00 +00:00")
         # Called once for XRP/BTC
         ohlcv_data = {
-            ('ETH/BTC', '1d'): ohlcv_history,
-            ('TKN/BTC', '1d'): ohlcv_history,
-            ('LTC/BTC', '1d'): ohlcv_history,
-            ('XRP/BTC', '1d'): ohlcv_history,
+            ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('LTC/BTC', '1d', CandleType.SPOT): ohlcv_history,
+            ('XRP/BTC', '1d', CandleType.SPOT): ohlcv_history,
         }
         mocker.patch('freqtrade.exchange.Exchange.refresh_latest_ohlcv', return_value=ohlcv_data)
         freqtrade.pairlists.refresh_pairlist()
@@ -1046,12 +1046,12 @@ def test_rangestabilityfilter_caching(mocker, markets, default_conf, tickers, oh
                           get_tickers=tickers
                           )
     ohlcv_data = {
-        ('ETH/BTC', '1d'): ohlcv_history,
-        ('TKN/BTC', '1d'): ohlcv_history,
-        ('LTC/BTC', '1d'): ohlcv_history,
-        ('XRP/BTC', '1d'): ohlcv_history,
-        ('HOT/BTC', '1d'): ohlcv_history,
-        ('BLK/BTC', '1d'): ohlcv_history,
+        ('ETH/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('TKN/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('LTC/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('XRP/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('HOT/BTC', '1d', CandleType.SPOT): ohlcv_history,
+        ('BLK/BTC', '1d', CandleType.SPOT): ohlcv_history,
     }
     mocker.patch.multiple(
         'freqtrade.exchange.Exchange',
