@@ -1,6 +1,5 @@
 # pragma pylint: disable=missing-docstring, protected-access, C0103
 import logging
-import warnings
 from base64 import urlsafe_b64encode
 from pathlib import Path
 
@@ -10,7 +9,7 @@ from pandas import DataFrame
 from freqtrade.exceptions import OperationalException
 from freqtrade.resolvers import StrategyResolver
 from freqtrade.strategy.interface import IStrategy
-from tests.conftest import log_has, log_has_re
+from tests.conftest import CURRENT_TEST_STRATEGY, log_has, log_has_re
 
 
 def test_search_strategy():
@@ -18,7 +17,7 @@ def test_search_strategy():
 
     s, _ = StrategyResolver._search_object(
         directory=default_location,
-        object_name='StrategyTestV2',
+        object_name=CURRENT_TEST_STRATEGY,
         add_source=True,
     )
     assert issubclass(s, IStrategy)
@@ -35,7 +34,7 @@ def test_search_all_strategies_no_failed():
     directory = Path(__file__).parent / "strats"
     strategies = StrategyResolver.search_all_objects(directory, enum_failed=False)
     assert isinstance(strategies, list)
-    assert len(strategies) == 4
+    assert len(strategies) == 5
     assert isinstance(strategies[0], dict)
 
 
@@ -43,10 +42,10 @@ def test_search_all_strategies_with_failed():
     directory = Path(__file__).parent / "strats"
     strategies = StrategyResolver.search_all_objects(directory, enum_failed=True)
     assert isinstance(strategies, list)
-    assert len(strategies) == 5
+    assert len(strategies) == 6
     # with enum_failed=True search_all_objects() shall find 2 good strategies
     # and 1 which fails to load
-    assert len([x for x in strategies if x['class'] is not None]) == 4
+    assert len([x for x in strategies if x['class'] is not None]) == 5
     assert len([x for x in strategies if x['class'] is None]) == 1
 
 
@@ -74,10 +73,10 @@ def test_load_strategy_base64(result, caplog, default_conf):
 
 
 def test_load_strategy_invalid_directory(result, caplog, default_conf):
-    default_conf['strategy'] = 'StrategyTestV2'
+    default_conf['strategy'] = 'StrategyTestV3'
     extra_dir = Path.cwd() / 'some/path'
     with pytest.raises(OperationalException):
-        StrategyResolver._load_strategy('StrategyTestV2', config=default_conf,
+        StrategyResolver._load_strategy(CURRENT_TEST_STRATEGY, config=default_conf,
                                         extra_dir=extra_dir)
 
     assert log_has_re(r'Path .*' + r'some.*path.*' + r'.* does not exist', caplog)
@@ -99,8 +98,10 @@ def test_load_strategy_noname(default_conf):
         StrategyResolver.load_strategy(default_conf)
 
 
-def test_strategy(result, default_conf):
-    default_conf.update({'strategy': 'StrategyTestV2'})
+@pytest.mark.filterwarnings("ignore:deprecated")
+@pytest.mark.parametrize('strategy_name', ['StrategyTestV2'])
+def test_strategy_pre_v3(result, default_conf, strategy_name):
+    default_conf.update({'strategy': strategy_name})
 
     strategy = StrategyResolver.load_strategy(default_conf)
     metadata = {'pair': 'ETH/BTC'}
@@ -116,17 +117,35 @@ def test_strategy(result, default_conf):
     df_indicators = strategy.advise_indicators(result, metadata=metadata)
     assert 'adx' in df_indicators
 
-    dataframe = strategy.advise_buy(df_indicators, metadata=metadata)
-    assert 'buy' in dataframe.columns
+    dataframe = strategy.advise_entry(df_indicators, metadata=metadata)
+    assert 'buy' not in dataframe.columns
+    assert 'enter_long' in dataframe.columns
 
-    dataframe = strategy.advise_sell(df_indicators, metadata=metadata)
-    assert 'sell' in dataframe.columns
+    dataframe = strategy.advise_exit(df_indicators, metadata=metadata)
+    assert 'sell' not in dataframe.columns
+    assert 'exit_long' in dataframe.columns
+
+
+def test_strategy_can_short(caplog, default_conf):
+    caplog.set_level(logging.INFO)
+    default_conf.update({
+        'strategy': CURRENT_TEST_STRATEGY,
+    })
+    strat = StrategyResolver.load_strategy(default_conf)
+    assert isinstance(strat, IStrategy)
+    default_conf['strategy'] = 'StrategyTestV3Futures'
+    with pytest.raises(ImportError, match=""):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['trading_mode'] = 'futures'
+    strat = StrategyResolver.load_strategy(default_conf)
+    assert isinstance(strat, IStrategy)
 
 
 def test_strategy_override_minimal_roi(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'minimal_roi': {
             "20": 0.1,
             "0": 0.5
@@ -143,7 +162,7 @@ def test_strategy_override_minimal_roi(caplog, default_conf):
 def test_strategy_override_stoploss(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'stoploss': -0.5
     })
     strategy = StrategyResolver.load_strategy(default_conf)
@@ -155,7 +174,7 @@ def test_strategy_override_stoploss(caplog, default_conf):
 def test_strategy_override_trailing_stop(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'trailing_stop': True
     })
     strategy = StrategyResolver.load_strategy(default_conf)
@@ -168,7 +187,7 @@ def test_strategy_override_trailing_stop(caplog, default_conf):
 def test_strategy_override_trailing_stop_positive(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'trailing_stop_positive': -0.1,
         'trailing_stop_positive_offset': -0.2
 
@@ -188,7 +207,7 @@ def test_strategy_override_timeframe(caplog, default_conf):
     caplog.set_level(logging.INFO)
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'timeframe': 60,
         'stake_currency': 'ETH'
     })
@@ -204,7 +223,7 @@ def test_strategy_override_process_only_new_candles(caplog, default_conf):
     caplog.set_level(logging.INFO)
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'process_only_new_candles': True
     })
     strategy = StrategyResolver.load_strategy(default_conf)
@@ -218,32 +237,32 @@ def test_strategy_override_order_types(caplog, default_conf):
     caplog.set_level(logging.INFO)
 
     order_types = {
-        'buy': 'market',
-        'sell': 'limit',
+        'entry': 'market',
+        'exit': 'limit',
         'stoploss': 'limit',
         'stoploss_on_exchange': True,
     }
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'order_types': order_types
     })
     strategy = StrategyResolver.load_strategy(default_conf)
 
     assert strategy.order_types
-    for method in ['buy', 'sell', 'stoploss', 'stoploss_on_exchange']:
+    for method in ['entry', 'exit', 'stoploss', 'stoploss_on_exchange']:
         assert strategy.order_types[method] == order_types[method]
 
     assert log_has("Override strategy 'order_types' with value in config file:"
-                   " {'buy': 'market', 'sell': 'limit', 'stoploss': 'limit',"
+                   " {'entry': 'market', 'exit': 'limit', 'stoploss': 'limit',"
                    " 'stoploss_on_exchange': True}.", caplog)
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
-        'order_types': {'buy': 'market'}
+        'strategy': CURRENT_TEST_STRATEGY,
+        'order_types': {'exit': 'market'}
     })
     # Raise error for invalid configuration
     with pytest.raises(ImportError,
-                       match=r"Impossible to load Strategy 'StrategyTestV2'. "
+                       match=r"Impossible to load Strategy '" + CURRENT_TEST_STRATEGY + "'. "
                              r"Order-types mapping is incomplete."):
         StrategyResolver.load_strategy(default_conf)
 
@@ -252,162 +271,166 @@ def test_strategy_override_order_tif(caplog, default_conf):
     caplog.set_level(logging.INFO)
 
     order_time_in_force = {
-        'buy': 'fok',
-        'sell': 'gtc',
+        'entry': 'fok',
+        'exit': 'gtc',
     }
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
         'order_time_in_force': order_time_in_force
     })
     strategy = StrategyResolver.load_strategy(default_conf)
 
     assert strategy.order_time_in_force
-    for method in ['buy', 'sell']:
+    for method in ['entry', 'exit']:
         assert strategy.order_time_in_force[method] == order_time_in_force[method]
 
     assert log_has("Override strategy 'order_time_in_force' with value in config file:"
-                   " {'buy': 'fok', 'sell': 'gtc'}.", caplog)
+                   " {'entry': 'fok', 'exit': 'gtc'}.", caplog)
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
-        'order_time_in_force': {'buy': 'fok'}
+        'strategy': CURRENT_TEST_STRATEGY,
+        'order_time_in_force': {'entry': 'fok'}
     })
     # Raise error for invalid configuration
     with pytest.raises(ImportError,
-                       match=r"Impossible to load Strategy 'StrategyTestV2'. "
-                             r"Order-time-in-force mapping is incomplete."):
+                       match=f"Impossible to load Strategy '{CURRENT_TEST_STRATEGY}'. "
+                             "Order-time-in-force mapping is incomplete."):
         StrategyResolver.load_strategy(default_conf)
 
 
-def test_strategy_override_use_sell_signal(caplog, default_conf):
+def test_strategy_override_use_exit_signal(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
     })
     strategy = StrategyResolver.load_strategy(default_conf)
-    assert strategy.use_sell_signal
-    assert isinstance(strategy.use_sell_signal, bool)
+    assert strategy.use_exit_signal
+    assert isinstance(strategy.use_exit_signal, bool)
     # must be inserted to configuration
-    assert 'use_sell_signal' in default_conf
-    assert default_conf['use_sell_signal']
+    assert 'use_exit_signal' in default_conf
+    assert default_conf['use_exit_signal']
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
-        'use_sell_signal': False,
+        'strategy': CURRENT_TEST_STRATEGY,
+        'use_exit_signal': False,
     })
     strategy = StrategyResolver.load_strategy(default_conf)
 
-    assert not strategy.use_sell_signal
-    assert isinstance(strategy.use_sell_signal, bool)
-    assert log_has("Override strategy 'use_sell_signal' with value in config file: False.", caplog)
+    assert not strategy.use_exit_signal
+    assert isinstance(strategy.use_exit_signal, bool)
+    assert log_has("Override strategy 'use_exit_signal' with value in config file: False.", caplog)
 
 
-def test_strategy_override_use_sell_profit_only(caplog, default_conf):
+def test_strategy_override_use_exit_profit_only(caplog, default_conf):
     caplog.set_level(logging.INFO)
     default_conf.update({
-        'strategy': 'StrategyTestV2',
+        'strategy': CURRENT_TEST_STRATEGY,
     })
     strategy = StrategyResolver.load_strategy(default_conf)
-    assert not strategy.sell_profit_only
-    assert isinstance(strategy.sell_profit_only, bool)
+    assert not strategy.exit_profit_only
+    assert isinstance(strategy.exit_profit_only, bool)
     # must be inserted to configuration
-    assert 'sell_profit_only' in default_conf
-    assert not default_conf['sell_profit_only']
+    assert 'exit_profit_only' in default_conf
+    assert not default_conf['exit_profit_only']
 
     default_conf.update({
-        'strategy': 'StrategyTestV2',
-        'sell_profit_only': True,
+        'strategy': CURRENT_TEST_STRATEGY,
+        'exit_profit_only': True,
     })
     strategy = StrategyResolver.load_strategy(default_conf)
 
-    assert strategy.sell_profit_only
-    assert isinstance(strategy.sell_profit_only, bool)
-    assert log_has("Override strategy 'sell_profit_only' with value in config file: True.", caplog)
+    assert strategy.exit_profit_only
+    assert isinstance(strategy.exit_profit_only, bool)
+    assert log_has("Override strategy 'exit_profit_only' with value in config file: True.", caplog)
 
 
 @pytest.mark.filterwarnings("ignore:deprecated")
-def test_deprecate_populate_indicators(result, default_conf):
+def test_missing_implements(default_conf, caplog):
+
     default_location = Path(__file__).parent / "strats"
-    default_conf.update({'strategy': 'TestStrategyLegacyV1',
+    default_conf.update({'strategy': 'StrategyTestV2',
                          'strategy_path': default_location})
-    strategy = StrategyResolver.load_strategy(default_conf)
-    with warnings.catch_warnings(record=True) as w:
-        # Cause all warnings to always be triggered.
-        warnings.simplefilter("always")
-        indicators = strategy.advise_indicators(result, {'pair': 'ETH/BTC'})
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        assert "deprecated - check out the Sample strategy to see the current function headers!" \
-            in str(w[-1].message)
+    StrategyResolver.load_strategy(default_conf)
 
-    with warnings.catch_warnings(record=True) as w:
-        # Cause all warnings to always be triggered.
-        warnings.simplefilter("always")
-        strategy.advise_buy(indicators, {'pair': 'ETH/BTC'})
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        assert "deprecated - check out the Sample strategy to see the current function headers!" \
-            in str(w[-1].message)
+    log_has_re(r"DEPRECATED: .*use_sell_signal.*use_exit_signal.", caplog)
 
-    with warnings.catch_warnings(record=True) as w:
-        # Cause all warnings to always be triggered.
-        warnings.simplefilter("always")
-        strategy.advise_sell(indicators, {'pair': 'ETH_BTC'})
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        assert "deprecated - check out the Sample strategy to see the current function headers!" \
-            in str(w[-1].message)
+    default_conf['trading_mode'] = 'futures'
+    with pytest.raises(OperationalException,
+                       match=r"DEPRECATED: .*use_sell_signal.*use_exit_signal."):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['trading_mode'] = 'spot'
+
+    default_location = Path(__file__).parent / "strats/broken_strats"
+    default_conf.update({'strategy': 'TestStrategyNoImplements',
+                         'strategy_path': default_location})
+    with pytest.raises(OperationalException,
+                       match=r"`populate_entry_trend` or `populate_buy_trend`.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['strategy'] = 'TestStrategyNoImplementSell'
+
+    with pytest.raises(OperationalException,
+                       match=r"`populate_exit_trend` or `populate_sell_trend`.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    # Futures mode is more strict ...
+    default_conf['trading_mode'] = 'futures'
+
+    with pytest.raises(OperationalException,
+                       match=r"`populate_exit_trend` must be implemented.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['strategy'] = 'TestStrategyNoImplements'
+    with pytest.raises(OperationalException,
+                       match=r"`populate_entry_trend` must be implemented.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['strategy'] = 'TestStrategyImplementCustomSell'
+    with pytest.raises(OperationalException,
+                       match=r"Please migrate your implementation of `custom_sell`.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['strategy'] = 'TestStrategyImplementBuyTimeout'
+    with pytest.raises(OperationalException,
+                       match=r"Please migrate your implementation of `check_buy_timeout`.*"):
+        StrategyResolver.load_strategy(default_conf)
+
+    default_conf['strategy'] = 'TestStrategyImplementSellTimeout'
+    with pytest.raises(OperationalException,
+                       match=r"Please migrate your implementation of `check_sell_timeout`.*"):
+        StrategyResolver.load_strategy(default_conf)
 
 
-@pytest.mark.filterwarnings("ignore:deprecated")
-def test_call_deprecated_function(result, monkeypatch, default_conf, caplog):
-    default_location = Path(__file__).parent / "strats"
+def test_call_deprecated_function(default_conf):
+    default_location = Path(__file__).parent / "strats/broken_strats/"
     del default_conf['timeframe']
     default_conf.update({'strategy': 'TestStrategyLegacyV1',
                          'strategy_path': default_location})
-    strategy = StrategyResolver.load_strategy(default_conf)
-    metadata = {'pair': 'ETH/BTC'}
-
-    # Make sure we are using a legacy function
-    assert strategy._populate_fun_len == 2
-    assert strategy._buy_fun_len == 2
-    assert strategy._sell_fun_len == 2
-    assert strategy.INTERFACE_VERSION == 1
-    assert strategy.timeframe == '5m'
-
-    indicator_df = strategy.advise_indicators(result, metadata=metadata)
-    assert isinstance(indicator_df, DataFrame)
-    assert 'adx' in indicator_df.columns
-
-    buydf = strategy.advise_buy(result, metadata=metadata)
-    assert isinstance(buydf, DataFrame)
-    assert 'buy' in buydf.columns
-
-    selldf = strategy.advise_sell(result, metadata=metadata)
-    assert isinstance(selldf, DataFrame)
-    assert 'sell' in selldf
+    with pytest.raises(OperationalException,
+                       match=r"Strategy Interface v1 is no longer supported.*"):
+        StrategyResolver.load_strategy(default_conf)
 
 
-def test_strategy_interface_versioning(result, monkeypatch, default_conf):
+def test_strategy_interface_versioning(result, default_conf):
     default_conf.update({'strategy': 'StrategyTestV2'})
     strategy = StrategyResolver.load_strategy(default_conf)
     metadata = {'pair': 'ETH/BTC'}
 
-    # Make sure we are using a legacy function
-    assert strategy._populate_fun_len == 3
-    assert strategy._buy_fun_len == 3
-    assert strategy._sell_fun_len == 3
     assert strategy.INTERFACE_VERSION == 2
 
     indicator_df = strategy.advise_indicators(result, metadata=metadata)
     assert isinstance(indicator_df, DataFrame)
     assert 'adx' in indicator_df.columns
 
-    buydf = strategy.advise_buy(result, metadata=metadata)
-    assert isinstance(buydf, DataFrame)
-    assert 'buy' in buydf.columns
+    enterdf = strategy.advise_entry(result, metadata=metadata)
+    assert isinstance(enterdf, DataFrame)
 
-    selldf = strategy.advise_sell(result, metadata=metadata)
-    assert isinstance(selldf, DataFrame)
-    assert 'sell' in selldf
+    assert 'buy' not in enterdf.columns
+    assert 'enter_long' in enterdf.columns
+
+    exitdf = strategy.advise_exit(result, metadata=metadata)
+    assert isinstance(exitdf, DataFrame)
+    assert 'sell' not in exitdf
+    assert 'exit_long' in exitdf
