@@ -10,7 +10,8 @@ from plotly.subplots import make_subplots
 from freqtrade.commands import start_plot_dataframe, start_plot_profit
 from freqtrade.configuration import TimeRange
 from freqtrade.data import history
-from freqtrade.data.btanalysis import create_cum_profit, load_backtest_data
+from freqtrade.data.btanalysis import load_backtest_data
+from freqtrade.data.metrics import create_cum_profit
 from freqtrade.exceptions import OperationalException
 from freqtrade.plot.plotting import (add_areas, add_indicators, add_profit, create_plotconfig,
                                      generate_candlestick_graph, generate_plot_filename,
@@ -157,7 +158,7 @@ def test_plot_trades(testdatadir, caplog):
     assert fig == fig1
     assert log_has("No trades found.", caplog)
     pair = "ADA/BTC"
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     trades = load_backtest_data(filename)
     trades = trades.loc[trades['pair'] == pair]
 
@@ -200,8 +201,10 @@ def test_generate_candlestick_graph_no_signals_no_trades(default_conf, mocker, t
     timerange = TimeRange(None, 'line', 0, -1000)
     data = history.load_pair_history(pair=pair, timeframe='1m',
                                      datadir=testdatadir, timerange=timerange)
-    data['buy'] = 0
-    data['sell'] = 0
+    data['enter_long'] = 0
+    data['exit_long'] = 0
+    data['enter_short'] = 0
+    data['exit_short'] = 0
 
     indicators1 = []
     indicators2 = []
@@ -222,8 +225,10 @@ def test_generate_candlestick_graph_no_signals_no_trades(default_conf, mocker, t
     assert row_mock.call_count == 2
     assert trades_mock.call_count == 1
 
-    assert log_has("No buy-signals found.", caplog)
-    assert log_has("No sell-signals found.", caplog)
+    assert log_has("No enter_long-signals found.", caplog)
+    assert log_has("No exit_long-signals found.", caplog)
+    assert log_has("No enter_short-signals found.", caplog)
+    assert log_has("No exit_short-signals found.", caplog)
 
 
 def test_generate_candlestick_graph_no_trades(default_conf, mocker, testdatadir):
@@ -249,7 +254,7 @@ def test_generate_candlestick_graph_no_trades(default_conf, mocker, testdatadir)
     assert fig.layout.title.text == pair
     figure = fig.layout.figure
 
-    assert len(figure.data) == 6
+    assert len(figure.data) == 8
     # Candlesticks are plotted first
     candles = find_trace_in_fig_data(figure.data, "Price")
     assert isinstance(candles, go.Candlestick)
@@ -257,15 +262,15 @@ def test_generate_candlestick_graph_no_trades(default_conf, mocker, testdatadir)
     volume = find_trace_in_fig_data(figure.data, "Volume")
     assert isinstance(volume, go.Bar)
 
-    buy = find_trace_in_fig_data(figure.data, "buy")
-    assert isinstance(buy, go.Scatter)
+    enter_long = find_trace_in_fig_data(figure.data, "enter_long")
+    assert isinstance(enter_long, go.Scatter)
     # All buy-signals should be plotted
-    assert int(data.buy.sum()) == len(buy.x)
+    assert int(data['enter_long'].sum()) == len(enter_long.x)
 
-    sell = find_trace_in_fig_data(figure.data, "sell")
-    assert isinstance(sell, go.Scatter)
+    exit_long = find_trace_in_fig_data(figure.data, "exit_long")
+    assert isinstance(exit_long, go.Scatter)
     # All buy-signals should be plotted
-    assert int(data.sell.sum()) == len(sell.x)
+    assert int(data['exit_long'].sum()) == len(exit_long.x)
 
     assert find_trace_in_fig_data(figure.data, "Bollinger Band")
 
@@ -294,7 +299,7 @@ def test_generate_plot_file(mocker, caplog):
 
 
 def test_add_profit(testdatadir):
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     bt_data = load_backtest_data(filename)
     timerange = TimeRange.parse_timerange("20180110-20180112")
 
@@ -314,7 +319,7 @@ def test_add_profit(testdatadir):
 
 
 def test_generate_profit_graph(testdatadir):
-    filename = testdatadir / "backtest-result_new.json"
+    filename = testdatadir / "backtest_results/backtest-result_new.json"
     trades = load_backtest_data(filename)
     timerange = TimeRange.parse_timerange("20180110-20180112")
     pairs = ["TRX/BTC", "XLM/BTC"]
@@ -452,7 +457,7 @@ def test_plot_profit(default_conf, mocker, testdatadir):
                        match=r"No trades found, cannot generate Profit-plot.*"):
         plot_profit(default_conf)
 
-    default_conf['exportfilename'] = testdatadir / "backtest-result_new.json"
+    default_conf['exportfilename'] = testdatadir / "backtest_results/backtest-result_new.json"
 
     plot_profit(default_conf)
 
